# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from pyrit.prompt_converter import PromptConverter


class StringJoinConverter(PromptConverter):

    def __init__(self, *, join_value="-"):
        self.join_value = join_value

    def convert(self, prompts: list[str]) -> list[str]:
        """
        Simple converter that uses str join for letters between. E.g. with a `-`
        it converts a prompt of `test` to `t-e-s-t`

        This can sometimes bypass LLM logic

        Args:
            prompt (str): The prompt to be converted.

        Returns:
            list[str]: The converted prompts.
        """
        return [self.join_value.join(prompt) for prompt in prompts]

    def is_one_to_one_converter(self) -> bool:
        return True
